#include "fs/devices/FramebufferDevice.h"
#include "video/Framebuffer.h"
#include <bits/ioctl-defs.h>
#include <luna/CString.h>

Result<void> FramebufferDevice::create()
{
    auto device = (SharedPtr<Device>)TRY(make_shared<FramebufferDevice>());
    return DeviceRegistry::register_special_device(DeviceRegistry::Framebuffer, 0, device, 0600);
}

Result<usize> FramebufferDevice::read(u8*, usize, usize) const
{
    return 0;
}

Result<usize> FramebufferDevice::write(const u8* buf, usize offset, usize length)
{
    if ((offset + length) > size()) length = size() - offset;

    memcpy(Framebuffer::ptr() + offset, buf, length);

    return length;
}

usize FramebufferDevice::size() const
{
    return Framebuffer::size();
}

bool FramebufferDevice::will_block_if_read() const
{
    return false;
}

Result<u64> FramebufferDevice::ioctl(int request, void*)
{
    switch (request)
    {
    case FB_GET_WIDTH: return (u64)Framebuffer::width();
    case FB_GET_HEIGHT: return (u64)Framebuffer::height();
    case FB_GET_SCANLINE: return (u64)Framebuffer::scanline();
    default: return err(EINVAL);
    }
}
