#pragma once
#include "api/Mouse.h"
#include "fs/devices/DeviceRegistry.h"
#include <luna/CircularQueue.h>

class MouseDevice : public Device
{
  public:
    // Initializer for DeviceRegistry.
    static Result<void> create();

    Result<usize> read(u8*, usize, usize) const override;

    Result<usize> write(const u8*, usize, usize) override;

    static void add_mouse_event(const moon::MousePacket& packet);

    bool will_block_if_read() const override;

    StringView device_path() const override
    {
        return "mouse";
    }

    virtual ~MouseDevice() = default;

  private:
    mutable CircularQueue<moon::MousePacket, 200> m_packet_queue;

    static SharedPtr<MouseDevice> s_mouse_device;
};
